#include "sounds-cd.h"

const unsigned char conmul4[16][16] = {
  { 0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0 },
  { 0x0,0x1,0x2,0x3,0x4,0x5,0x6,0x7,0x8,0x9,0xa,0xb,0xc,0xd,0xe,0xf },
  { 0x0,0x2,0x3,0x1,0x8,0xa,0xb,0x9,0xc,0xe,0xf,0xd,0x4,0x6,0x7,0x5 },
  { 0x0,0x3,0x1,0x2,0xc,0xf,0xd,0xe,0x4,0x7,0x5,0x6,0x8,0xb,0x9,0xa },
  { 0x0,0x4,0x8,0xc,0x6,0x2,0xe,0xa,0xb,0xf,0x3,0x7,0xd,0x9,0x5,0x1 },
  { 0x0,0x5,0xa,0xf,0x2,0x7,0x8,0xd,0x3,0x6,0x9,0xc,0x1,0x4,0xb,0xe },
  { 0x0,0x6,0xb,0xd,0xe,0x8,0x5,0x3,0x7,0x1,0xc,0xa,0x9,0xf,0x2,0x4 },
  { 0x0,0x7,0x9,0xe,0xa,0xd,0x3,0x4,0xf,0x8,0x6,0x1,0x5,0x2,0xc,0xb },
  { 0x0,0x8,0xc,0x4,0xb,0x3,0x7,0xf,0xd,0x5,0x1,0x9,0x6,0xe,0xa,0x2 },
  { 0x0,0x9,0xe,0x7,0xf,0x6,0x1,0x8,0x5,0xc,0xb,0x2,0xa,0x3,0x4,0xd },
  { 0x0,0xa,0xf,0x5,0x3,0x9,0xc,0x6,0x1,0xb,0xe,0x4,0x2,0x8,0xd,0x7 },
  { 0x0,0xb,0xd,0x6,0x7,0xc,0xa,0x1,0x9,0x2,0x4,0xf,0xe,0x5,0x3,0x8 },
  { 0x0,0xc,0x4,0x8,0xd,0x1,0x9,0x5,0x6,0xa,0x2,0xe,0xb,0x7,0xf,0x3 },
  { 0x0,0xd,0x6,0xb,0x9,0x4,0xf,0x2,0xe,0x3,0x8,0x5,0x7,0xa,0x1,0xc },
  { 0x0,0xe,0x7,0x9,0x5,0xb,0x2,0xc,0xa,0x4,0xd,0x3,0xf,0x1,0x8,0x6 },
  { 0x0,0xf,0x5,0xa,0x1,0xe,0x4,0xb,0x2,0xd,0x7,0x8,0x3,0xc,0x6,0x9 }
};

static unsigned char
conmul8 (unsigned char a, unsigned char b)
{
  unsigned char a0, a1, b0, b1, t, c0, c1;

  a0 = a&0xf;  a1 = (a>>4)&0xf;
  b0 = b&0xf;  b1 = (b>>4)&0xf;
  t = conmul4[a1][b1];
  c0 = conmul4[a0][b0] ^ conmul4[0x8][t];
  c1 = conmul4[a0][b1] ^ conmul4[a1][b0] ^ t;
  return c0|(c1<<4);
}

static unsigned char
conmul8_80 (unsigned char a)
{
  unsigned char a0, a1, t, c0, c1;

  a0 = a&0xf;  a1 = (a>>4)&0xf;
  t = conmul4[0x8][a1];
  c0 = conmul4[0x8][t];
  c1 = conmul4[0x8][a0] ^ t;
  return c0|(c1<<4);
}

static unsigned short
conmul16 (unsigned short a, unsigned short b)
{
  unsigned char a0, a1, b0, b1, t, c0, c1;

  a0 = a&0xff;  a1 = (a>>8)&0xff;
  b0 = b&0xff;  b1 = (b>>8)&0xff;
  t = conmul8 (a1, b1);
  c0 = conmul8 (a0, b0) ^ conmul8_80 (t);
  c1 = conmul8 (a0, b1) ^ conmul8 (a1, b0) ^ t;
  return ((unsigned short)c0)|(((unsigned short)c1)<<8);
}

#if defined(__GNUC__)
static char
conway_mul_data (void *junk __attribute__((__unused__)), unsigned long time,
		 short *left, short *right)
#else
static char
conway_mul_data (void *junk, unsigned long time,
		 short *left, short *right)
#endif
{
  unsigned short x, y;

  x = time&0xffff;
  y = conmul16 (x, 0x100);
  *left = y - 0x8000;
  *right = y - 0x8000;
  return ( time < 8*0x10000UL );
}

int
main (void)
{
  record_track (conway_mul_data, 0);
  return 0;
}
